cpdef object read_json(
    bytes binary_data,
    object pgoid_function=*,
    object buffer_object=*,
    object pgoid=*,
)
cpdef bytes write_json(
    object dtype_value,
    object pgoid_function=*,
    object buffer_object=*,
    object pgoid=*,
)
